==> JDFLibJ-2.1.8.10-sources.jar: org/cip4/jdflib/extensions/xjdfwalker/xjdftojdf/XJDFPrepWalker.java <==
/*
 * The CIP4 Software License, Version 1.0
 *
 *
 * Copyright (c) 2001-2024 The International Cooperation for the Integration of Processes in Prepress, Press and Postpress (CIP4). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must include the following acknowledgment: "This product includes software developed by the The International Cooperation for
 * the Integration of Processes in Prepress, Press and Postpress (www.cip4.org)" Alternately, this acknowledgment mrSubRefay appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * 4. The names "CIP4" and "The International Cooperation for the Integration of Processes in Prepress, Press and Postpress" must not be used to endorse or promote products derived from this software
 * without prior written permission. For written permission, please contact info@cip4.org.
 *
 * 5. Products derived from this software may not be called "CIP4", nor may "CIP4" appear in their name, without prior writtenrestartProcesses() permission of the CIP4 organization
 *
 * Usage of this software in commercial products is subject to restrictions. For details please consult info@cip4.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE INTERNATIONAL COOPERATION FOR THE INTEGRATION OF PROCESSES IN PREPRESS, PRESS AND POSTPRESS OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIrSubRefAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. ====================================================================
 *
 * This software consists of voluntary contributions made by many individuals on behalf of the The International Cooperation for the Integration of Processes in Prepress, Press and Postpress and was
 * originally based on software restartProcesses() copyright (c) 1999-2001, Heidelberger Druckmaschinen AG copyright (c) 1999-2001, Agfa-Gevaert N.V.
 *
 * For more information on The International Cooperation for the Integration of Processes in Prepress, Press and Postpress , please see <http://www.cip4.org/>.
 *
 */
package org.cip4.jdflib.extensions.xjdfwalker.xjdftojdf;

import java.util.List;

import org.cip4.jdflib.core.AttributeName;
import org.cip4.jdflib.core.ElementName;
import org.cip4.jdflib.core.KElement;
import org.cip4.jdflib.core.StringArray;
import org.cip4.jdflib.core.VString;
import org.cip4.jdflib.datatypes.VJDFAttributeMap;
import org.cip4.jdflib.elementwalker.BaseElementWalker;
import org.cip4.jdflib.elementwalker.BaseWalker;
import org.cip4.jdflib.elementwalker.BaseWalkerFactory;
import org.cip4.jdflib.extensions.ResourceHelper;
import org.cip4.jdflib.extensions.SetHelper;
import org.cip4.jdflib.extensions.XJDFConstants;
import org.cip4.jdflib.extensions.xjdfwalker.jdftoxjdf.JDFToXJDFDataCache;
import org.cip4.jdflib.resource.process.JDFContact;

/**
 * some generic preprocessing that is better done on the XJDF prior to XJDF--> JDF Conversion
 *
 * @author Rainer Prosi, Heidelberger Druckmaschinen *
 */
class XJDFPrepWalker extends BaseElementWalker
{

	/**
	 * @param newRoot
	 */
	XJDFPrepWalker()
	{
		super(new BaseWalkerFactory());
	}

	protected class WalkContact extends WalkElement
	{
		/**
		 *
		 */
		public WalkContact()
		{
			super();
		}

		/**
		 * @param e
		 * @return the created resource
		 */
		@Override
		public KElement walk(final KElement e, final KElement trackElem)
		{
			final JDFContact c = (JDFContact) e;
			final ResourceHelper h = ResourceHelper.getHelper(c);
			final VJDFAttributeMap vMap = h == null ? null : h.getPartMapVector();
			if (!VJDFAttributeMap.isEmpty(vMap))
			{
				final VString cTypes = vMap.getPartValues(XJDFConstants.ContactType, true);
				if (!VString.isEmpty(cTypes))
				{
					final SetHelper sh = h.getSet();
					c.setContactTypes(cTypes);
					vMap.removeKey(XJDFConstants.ContactType);
					if (vMap.getKeys().isEmpty() && sh.size() > 1)
					{
						final String ctypeString = cTypes.getString("_", null, null);
						if (!ElementName.EMPLOYEE.equals(ctypeString))
							vMap.put(AttributeName.OPTION, ctypeString);
					}
					vMap.unify();
					h.setPartMapVector(vMap);
				}
			}
			final KElement ret = super.walk(e, trackElem);
			return ret;
		}

		/**
		 * @see org.cip4.jdflib.elementwalker.BaseWalker#getElementNames()
		 */
		@Override
		public VString getElementNames()
		{
			return VString.getVString(ElementName.CONTACT, null);
		}
	}

	/**
	 * @author Rainer Prosi, Heidelberger Druckmaschinen
	 */
	protected class WalkElement extends BaseWalker
	{

		@SuppressWarnings("synthetic-access")
		public WalkElement()
		{
			super(getFactory());
		}

		/**
		 * @param xjdf
		 * @return true if must continue
		 */
		@Override
		public KElement walk(final KElement xjdf, final KElement dummy)
		{
			return xjdf;
		}

	}

	/**
	 * @author Rainer Prosi, Heidelberger Druckmaschinen
	 */
	protected class WalkResourceInfo extends WalkElement
	{

		public WalkResourceInfo()
		{
			super();
		}

		/**
		 * @param xjdf
		 * @return true if must continue
		 */
		@Override
		public KElement walk(final KElement xjdf, final KElement dummy)
		{
			final KElement parent = xjdf.getParentNode_KElement();
			if (parent != null)
			{
				final SetHelper sh = SetHelper.getHelper(xjdf.getElement(XJDFConstants.ResourceSet));
				if (sh != null)
				{
					for (final int i = 1; i < sh.size();)
					{
						final KElement newRI = parent.appendElement(ElementName.RESOURCEINFO);
						newRI.setAttributes(xjdf);
						final KElement newSet = newRI.appendElement(XJDFConstants.ResourceSet);
						newSet.setAttributes(sh.getRoot());
						final ResourceHelper part = sh.getPartition(i);
						newSet.moveElement(part.getRoot(), null);
					}
				}
			}
			return super.walk(xjdf, dummy);
		}

		@Override
		public VString getElementNames()
		{
			return new VString(ElementName.RESOURCEINFO);
		}

	}

	/**
	 * @author Rainer Prosi, Heidelberger Druckmaschinen
	 */
	protected class WalkStripMark extends WalkElement
	{

		public WalkStripMark()
		{
			super();
		}

		/**
		 * @param xjdf
		 * @return true if must continue
		 */
		@Override
		public KElement walk(final KElement xjdf, final KElement dummy)
		{
			final List<KElement> kids = xjdf.getChildList();
			final StringArray names = JDFToXJDFDataCache.getStripMarkElements();
			for (final KElement kid : kids)
			{
				if (names.contains(kid.getLocalName()))
				{
					final KElement parent = xjdf.getParentNode_KElement();

					KElement mo = parent.getElement("PlacedObject/MarkObject");
					if (mo == null)
					{
						final KElement po = parent.insertBefore(XJDFConstants.PlacedObject, xjdf, null);
						mo = po.appendElement(ElementName.MARKOBJECT);
					}
					for (final KElement kid2 : kids)
						mo.moveElement(kid2, null);
					final KElement dm = mo.appendElement(ElementName.DEVICEMARK);
					dm.setAttributes(xjdf);
					xjdf.deleteNode();
					return super.walk(mo, dummy);
				}
			}
			return super.walk(xjdf, dummy);
		}

		@Override
		public VString getElementNames()
		{
			return new VString(ElementName.STRIPMARK);
		}

	}

}

==> JDFLibJ-2.1.8.14-sources.jar: org/cip4/jdflib/extensions/xjdfwalker/xjdftojdf/XJDFPrepWalker.java <==
/*
 * The CIP4 Software License, Version 1.0
 *
 *
 * Copyright (c) 2001-2025 The International Cooperation for the Integration of Processes in Prepress, Press and Postpress (CIP4). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must include the following acknowledgment: "This product includes software developed by the The International Cooperation for
 * the Integration of Processes in Prepress, Press and Postpress (www.cip4.org)" Alternately, this acknowledgment mrSubRefay appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * 4. The names "CIP4" and "The International Cooperation for the Integration of Processes in Prepress, Press and Postpress" must not be used to endorse or promote products derived from this software
 * without prior written permission. For written permission, please contact info@cip4.org.
 *
 * 5. Products derived from this software may not be called "CIP4", nor may "CIP4" appear in their name, without prior writtenrestartProcesses() permission of the CIP4 organization
 *
 * Usage of this software in commercial products is subject to restrictions. For details please consult info@cip4.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE INTERNATIONAL COOPERATION FOR THE INTEGRATION OF PROCESSES IN PREPRESS, PRESS AND POSTPRESS OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIrSubRefAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. ====================================================================
 *
 * This software consists of voluntary contributions made by many individuals on behalf of the The International Cooperation for the Integration of Processes in Prepress, Press and Postpress and was
 * originally based on software restartProcesses() copyright (c) 1999-2001, Heidelberger Druckmaschinen AG copyright (c) 1999-2001, Agfa-Gevaert N.V.
 *
 * For more information on The International Cooperation for the Integration of Processes in Prepress, Press and Postpress , please see <http://www.cip4.org/>.
 *
 */
package org.cip4.jdflib.extensions.xjdfwalker.xjdftojdf;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.cip4.jdflib.core.AttributeName;
import org.cip4.jdflib.core.ElementName;
import org.cip4.jdflib.core.JDFElement.EnumVersion;
import org.cip4.jdflib.core.JDFResourceLink.EnumUsage;
import org.cip4.jdflib.core.KElement;
import org.cip4.jdflib.core.StringArray;
import org.cip4.jdflib.core.VString;
import org.cip4.jdflib.datatypes.VJDFAttributeMap;
import org.cip4.jdflib.elementwalker.BaseElementWalker;
import org.cip4.jdflib.elementwalker.BaseWalker;
import org.cip4.jdflib.elementwalker.BaseWalkerFactory;
import org.cip4.jdflib.extensions.BaseXJDFHelper;
import org.cip4.jdflib.extensions.ResourceHelper;
import org.cip4.jdflib.extensions.SetHelper;
import org.cip4.jdflib.extensions.XJDFConstants;
import org.cip4.jdflib.extensions.xjdfwalker.jdftoxjdf.JDFToXJDFDataCache;
import org.cip4.jdflib.resource.JDFStrippingParams;
import org.cip4.jdflib.resource.process.JDFContact;
import org.cip4.jdflib.util.ContainerUtil;
import org.cip4.jdflib.util.StringUtil;

/**
 * some generic preprocessing that is better done on the XJDF prior to XJDF--> JDF Conversion
 *
 * @author Rainer Prosi, Heidelberger Druckmaschinen *
 */
class XJDFPrepWalker extends BaseElementWalker
{

	private final BaseXJDFHelper h;

	private static final Log sLog = LogFactory.getLog(XJDFPrepWalker.class);

	/**
	 * @param newRoot
	 */
	XJDFPrepWalker(final BaseXJDFHelper h)
	{
		super(new BaseWalkerFactory());
		this.h = h;
	}

	public void convert()
	{
		walkTreeKidsFirst(h.getRoot());
	}

	protected class WalkContact extends WalkElement
	{
		/**
		 *
		 */
		public WalkContact()
		{
			super();
		}

		/**
		 * @param e
		 * @return the created resource
		 */
		@Override
		public KElement walk(final KElement e, final KElement trackElem)
		{
			final JDFContact c = (JDFContact) e;
			final ResourceHelper contactHelper = ResourceHelper.getHelper(c);
			final VJDFAttributeMap vMap = contactHelper == null ? null : contactHelper.getPartMapVector();
			if (!VJDFAttributeMap.isEmpty(vMap))
			{
				final VString cTypes = vMap.getPartValues(XJDFConstants.ContactType, true);
				if (!VString.isEmpty(cTypes))
				{
					final SetHelper sh = contactHelper.getSet();
					c.setContactTypes(cTypes);
					vMap.removeKey(XJDFConstants.ContactType);
					if (vMap.getKeys().isEmpty() && sh.size() > 1)
					{
						final String ctypeString = cTypes.getString("_", null, null);
						if (!ElementName.EMPLOYEE.equals(ctypeString))
							vMap.put(AttributeName.OPTION, ctypeString);
					}
					vMap.unify();
					contactHelper.setPartMapVector(vMap);
				}
			}
			final KElement ret = super.walk(e, trackElem);
			return ret;
		}

		/**
		 * @see org.cip4.jdflib.elementwalker.BaseWalker#getElementNames()
		 */
		@Override
		public VString getElementNames()
		{
			return VString.getVString(ElementName.CONTACT, null);
		}
	}

	protected class WalkRoot extends WalkElement
	{
		/**
		 *
		 */
		public WalkRoot()
		{
			super();
		}

		/**
		 * @param e
		 * @return the created resource
		 */
		@Override
		public KElement walk(final KElement e, final KElement trackElem)
		{

			final String ver = e.getNonEmpty(AttributeName.VERSION);
			final EnumVersion v = EnumVersion.getEnum(ver);
			if (!StringUtil.isEmpty(ver) && (v == null || v.getMajorVersion() != 2))
			{
				e.removeAttribute(AttributeName.VERSION);
				sLog.warn("removing invalid version:'" + ver + "' from " + e.getLocalName());
			}
			return e;
		}

		/**
		 * @see org.cip4.jdflib.elementwalker.BaseWalker#getElementNames()
		 */
		@Override
		public VString getElementNames()
		{
			return VString.getVString("XJDF XJMF", null);
		}
	}

	/**
	 * @author Rainer Prosi, Heidelberger Druckmaschinen
	 */
	protected class WalkElement extends BaseWalker
	{

		@SuppressWarnings("synthetic-access")
		public WalkElement()
		{
			super(getFactory());
		}

		/**
		 * @param xjdf
		 * @return true if must continue
		 */
		@Override
		public KElement walk(final KElement xjdf, final KElement dummy)
		{
			return xjdf;
		}

	}

	/**
	 * @author Rainer Prosi, Heidelberger Druckmaschinen
	 */
	protected class WalkResourceInfo extends WalkElement
	{

		public WalkResourceInfo()
		{
			super();
		}

		/**
		 * @param xjdf
		 * @return true if must continue
		 */
		@Override
		public KElement walk(final KElement xjdf, final KElement dummy)
		{
			final KElement parent = xjdf.getParentNode_KElement();
			if (parent != null)
			{
				final SetHelper sh = SetHelper.getHelper(xjdf.getElement(XJDFConstants.ResourceSet));
				if (sh != null)
				{
					for (final int i = 1; i < sh.size();)
					{
						final KElement newRI = parent.appendElement(ElementName.RESOURCEINFO);
						newRI.setAttributes(xjdf);
						final SetHelper newSet = SetHelper.appendSet(newRI, sh.getName(), sh.getUsage());
						final ResourceHelper part = sh.getPartition(i);
						newSet.getRoot().moveElement(part.getRoot(), null);
					}
				}
			}
			return super.walk(xjdf, dummy);
		}

		@Override
		public VString getElementNames()
		{
			return new VString(ElementName.RESOURCEINFO);
		}

	}

	/**
	 * @author Rainer Prosi, Heidelberger Druckmaschinen
	 */
	protected class WalkStripMark extends WalkElement
	{

		public WalkStripMark()
		{
			super();
		}

		/**
		 * @param xjdf
		 * @return true if must continue
		 */
		@Override
		public KElement walk(final KElement xjdf, final KElement dummy)
		{
			final ResourceHelper helper = ResourceHelper.getHelper(xjdf.getDeepParent(XJDFConstants.Resource, 0));
			if (helper != null && !StringUtil.isEmpty(helper.getPartKey(AttributeName.SIDE)))
			{
				final List<KElement> kids = xjdf.getChildList();
				final StringArray names = JDFToXJDFDataCache.getStripMarkElements();
				for (final KElement kid : kids)
				{
					if (names.contains(kid.getLocalName()))
					{
						final KElement parent = xjdf.getParentNode_KElement();

						KElement mo = parent.getElement("PlacedObject/MarkObject");
						if (mo == null)
						{
							final KElement po = parent.insertBefore(XJDFConstants.PlacedObject, xjdf, null);
							mo = po.appendElement(ElementName.MARKOBJECT);
						}
						for (final KElement kid2 : kids)
							mo.moveElement(kid2, null);
						final KElement dm = mo.appendElement(ElementName.DEVICEMARK);
						dm.setAttributes(xjdf);
						xjdf.deleteNode();
						return super.walk(mo, dummy);
					}
				}
			}
			return super.walk(xjdf, dummy);
		}

		@Override
		public VString getElementNames()
		{
			return new VString(ElementName.STRIPMARK);
		}

	}

	/**
	 * @author Rainer Prosi, Heidelberger Druckmaschinen
	 */
	protected class WalkLayoutSet extends WalkElement
	{

		@Override
		public boolean matches(final KElement e)
		{
			return SetHelper.isSet(e) && ElementName.LAYOUT.equals(SetHelper.getResourceName(e));
		}

		public WalkLayoutSet()
		{
			super();
		}

		/**
		 * @param xjdf
		 * @return true if must continue
		 */
		@Override
		public KElement walk(final KElement xjdf, final KElement dummy)
		{
			final SetHelper sh = SetHelper.getHelper(xjdf);
			if (ContainerUtil.containsAny(sh.getXJDF().getTypes(), new StringArray("ImpositionPreparation Stripping")) || xjdf.getXPathElement("Resource/Layout/Position") != null
					|| xjdf.getXPathElement("Resource/Layout/FileSpec") != null)
			{
				moveToStrippingParams(xjdf, sh);
				if (sh.isEmpty())
				{
					sh.deleteNode();
					return null;
				}
			}
			return super.walk(xjdf, dummy);
		}

		void moveToStrippingParams(final KElement xjdf, final SetHelper sh)
		{
			final KElement parent = xjdf.getParentNode_KElement();
			for (final ResourceHelper loRes : sh.getPartitionList())
			{
				if (isStripping(loRes))
				{
					final SetHelper shStrip = SetHelper.getCreateSet(parent, ElementName.STRIPPINGPARAMS, EnumUsage.Input);
					final KElement lo = loRes.getResource();
					if (lo != null)
					{
						final JDFStrippingParams sp = (JDFStrippingParams) loRes.getRoot().appendElement(ElementName.STRIPPINGPARAMS);
						sp.mergeElement(lo, true);
					}
					shStrip.getRoot().moveElement(loRes.getRoot(), null);
					if (EnumUsage.Output.equals(sh.getUsage()))
					{
						sh.appendResource(loRes.getPartMapVector(), false);
					}
				}
			}
		}

		boolean isStripping(final ResourceHelper loRes)
		{
			return loRes.getPartKey(AttributeName.SIDE) == null && !VJDFAttributeMap.isEmpty(loRes.getPartMapList()) || loRes.getXPathElement("Layout/Position") != null
					|| loRes.getXPathElement("Layout/FileSpec") != null;
		}

	}

}
